-- @release $Id: Priority.lua 1 2016-08-13 18:29:06Z reighnman $

if not ShamAndAwe then return end

local L = LibStub("AceLocale-3.0"):GetLocale("ShamAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
if not C then
  ShamAndAwe:SetConstants()
end
local LBF = LibStub("LibButtonFacade", true)
local LBFgroup = nil
local _, _, _, clientVersion = GetBuildInfo()
local minaoetargets = 3
--if LBF then
-- local LBFgroup = LBF:Group("ShamAndAwe")
--end
ShamAndAwe.PriorityFrame = CreateFrame("Button", "SAA_PriorityFrame", UIParent)
ShamAndAwe.PriorityFrame.cooldown = CreateFrame("Cooldown", "SAA_PriorityFrameCooldown", ShamAndAwe.PriorityFrame, "CooldownFrameTemplate") --Inherit from WoW template (texture not necessary)

---------------------------
-- Local Variables
---------------------------

ShamAndAwe.priorityTable = {}
ShamAndAwe.priorityTable.name = {}
ShamAndAwe.priorityTable.icon = {}
ShamAndAwe.priorityTable.test = {}

-- table.foreach(ShamAndAwe.priorityTable.name, print)

--------------------------
-- Helper Functions
--------------------------
-- returns : true if spellname is available in less than priority.cooldown /run print(ShamAndAwe:SpellCooldown(ShamAndAwe.constants["Mortal Strike ID"]))
-- or if spellname is currently activated
-- , remaining cooldown, 0 if ready to be cast
function ShamAndAwe:SpellAvailable(spellid)
  if ShamAndAwe.db.char.priority.hideImmune and self:IsTargetImmune(spellid) then
    return false
  end
  local start, duration = GetSpellCooldown(spellid)
  if start then
    local timeleft = start + duration - GetTime()
    return timeleft <= ShamAndAwe.db.char.priority.cooldown, duration, timeleft
  end
  return false, 999
end

function ShamAndAwe:SpellReady(spellid)
  if ShamAndAwe.db.char.priority.hideImmune and self:IsTargetImmune(spellid) then
    return false
  end
  local available = GetSpellInfo(GetSpellInfo(spellid))
  local usable = IsUsableSpell(spellid)
  if available and usable then
    return true, usable
  end
  return false, false
end

function ShamAndAwe:SpellCooldown(spellid) --/run print(GetSpellCooldown(ShamAndAwe.constants["Bladestorm ID"]))
  local start, duration = GetSpellCooldown(spellid)
  if start then
    local timeleft = start + duration - GetTime()
    return duration, timeleft
  end
  return false, 999
end

function ShamAndAwe:SpellCharges(spellid)
  local currentCharges, maxCharges, cooldownStart, cooldownDuration = GetSpellCharges(spellid)

  if maxCharges then
    local timeleft = cooldownStart + cooldownDuration - GetTime()
    local calc = (1-(timeleft/cooldownDuration))+currentCharges
    --local final = string.format("%.2f", calc)
    if currentCharges ~= maxCharges then
      return true, currentCharges, maxCharges, calc
    else
      return false, currentCharges, maxCharges, 0
    end
  else
    return false, 0, 0, 0
  end
end

function ShamAndAwe:PrintPriorities()
  for index = 1, 30 do
    ShamAndAwe:Print(ShamAndAwe.db.char.priority.prOption[index])
  end
end

function ShamAndAwe:IsBoss()
  local mylevel = UnitLevel("player")
  local bosslevel = mylevel + 3
  if UnitLevel("target") >= bosslevel or UnitClassification("target") == "worldboss" then
    return true
  end
end

function ShamAndAwe:GCDAvailable()
  local startTime, gcdDuration = GetSpellCooldown(ShamAndAwe.constants["SpellCoolDown"])
  local gcdTime = startTime + gcdDuration - GetTime()
  return gcdTime <= ShamAndAwe.db.char.priority.cooldown
end

function ShamAndAwe:IsOffGCD()
  local startTime, gcdDuration = GetSpellCooldown(ShamAndAwe.constants["SpellCoolDown"])
  local gcdTime = startTime + gcdDuration - GetTime()
  if gcdTime > 0 then
    return true
  else
    return false
  end
end

function ShamAndAwe:GetDebuffInfo(debuff)
  local index = 1
  while UnitDebuff("target", index) do
    local name, _, _, count, _, _, debuffExpires, unitCaster, _, _, id = UnitDebuff("target", index)
    local isMine = unitCaster == "player"
    if id == debuff and isMine then
      local duration = debuffExpires - GetTime()
      return duration > ShamAndAwe.db.char.priority.cooldown, duration, count, debuffExpires
    end
    index = index + 1
  end
  return false, 0, 0, 0
end

function ShamAndAwe:GetBuffInfo(buff)
  local index = 1
  while UnitBuff("player", index) do
    local name, _, _, count, _, _, buffExpires, _, _, _, id = UnitBuff("player", index)
    if id == buff then
      local duration = buffExpires - GetTime()
      return duration > ShamAndAwe.db.char.priority.cooldown, duration, count
    end
    index = index + 1
  end
  return false, 0, 0
end

function ShamAndAwe:IsTargetImmune(spell)
  local target = UnitName("target") or ""
  if spell and ShamAndAwe.db.char.immuneTargets[target.."_"..spell] then
    return true
  end
  return false
end

local minRange = 0
local items = {
  {5, 37727}, -- Ruby Acorn
  {6, 63427}, -- Worgsaw
  {8, 34368}, -- Attuned Crystal Cores /run print(IsItemInRange("Attuned Crystal Cores", "TARGET"))
  {10, 32321}, -- Sparrowhawk Net
  {15, 33069}, -- Sturdy Rope
  {20, 10645}, -- Gnomish Death Ray
  {25, 24268}, -- Netherweave Net
  {30, 835}, -- Large Rope Net
  {35, 24269}, -- Heavy Netherweave Net
  {40, 28767}, -- The Decapitator
  {45, 23836}, -- Goblin Rocket Launcher
  {50, 116139}, -- Haunting Memento
  {60, 32825}, -- Soul Cannon
  {70, 41265}, -- Eyesore Blaster
  {80, 35278}, -- Reinforced Net
  {100, 33119}, -- Malister's Frost Wand
}

function ShamAndAwe:CountNameplates(maxRange)
  local last = result or 0
  local counter = 0

  for i = 1, 40 do
    local unit = "nameplate"..i
    if UnitExists(unit) and not UnitIsFriend("player", unit) then
      local min, max = ShamAndAwe:GetRange(unit)
      if min and max and min >= minRange and max <= maxRange then
        counter = counter + 1
      end
    end
  end
  if counter > 0 then
    return counter
  else
    return 0
  end
end

function ShamAndAwe:GetRange(unit)
  if not UnitExists(unit) or UnitIsFriend("player", unit) then
    return nil
  end

  local minRange, maxRange = 0, 0
  for _, items in ipairs(items) do
    local range, item = items[1], items[2]
    if IsItemInRange("item:"..item, unit) then
      maxRange = range
      break
    end
    minRange = range
  end
  return minRange, maxRange
end

------------------
-- Misc Functions
------------------

function ShamAndAwe:EstimateIgnorePain()
  local versatilityDamageBonus = GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE) + GetVersatilityBonus(CR_VERSATILITY_DAMAGE_DONE);
  local versatilityDamangeBonusPerc = (1 + versatilityDamageBonus * 10 ^ -2)
  local curMaelstrom = UnitPower("player", "mana")

  if curMaelstrom >= 20 then
    local _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, currentIP = UnitBuff("Player","Ignore Pain")
    if not currentIP then
      currentIP = 0
    end
    local _, STR, _, _ = UnitStat("player", 1)
    local APbase, APposBuff, APnegBuff = UnitAttackPower("player")
    local APfinal = APbase + APposBuff + APnegBuff
    local ipMaxMaelstrom = 60
    return floor((7/15*min(curMaelstrom,ipMaxMaelstrom)*APfinal)*versatilityDamangeBonusPerc*ShamAndAwe.db.char.ipMultiplier*.9+currentIP)
  else
    return 0
  end
end

------------------
-- Priorities
------------------

function ShamAndAwe:SetPriorityTable()
  ShamAndAwe.priorityTable.name["none"] = L["None"]
  _, _, ShamAndAwe.priorityTable.icon["none"] = "Interface/Tooltips/UI-Tooltip-Background"
  ShamAndAwe.priorityTable.test["none"] = function () return false end
end

--ShamAndAwe

ShamAndAwe.priorityTable.name["fs"] = ShamAndAwe.constants["Feral Spirit"]
ShamAndAwe.priorityTable.icon["fs"] = ShamAndAwe.constants["Feral Spirit Icon"]
ShamAndAwe.priorityTable.test["fs"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Feral Spirit ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Feral Spirit ID"])
  end
end

ShamAndAwe.priorityTable.name["fs_boss"] = "Feral Spirit (if=boss)"
ShamAndAwe.priorityTable.icon["fs_boss"] = ShamAndAwe.constants["Feral Spirit Icon"]
ShamAndAwe.priorityTable.test["fs_boss"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Feral Spirit ID"]) and ShamAndAwe:IsBoss() then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Feral Spirit ID"])
  end
end

ShamAndAwe.priorityTable.name["bf_nbf"] = "Boulderfist (if=!buff.boulderfist)"
ShamAndAwe.priorityTable.icon["bf_nbf"] = ShamAndAwe.constants["Boulderfist Icon"]
ShamAndAwe.priorityTable.test["bf_nbf"] =
function ()
  if ShamAndAwe.db.char.talBoulderfist then
    if ShamAndAwe:SpellReady(ShamAndAwe.constants["Boulderfist ID"]) then
      local BF = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Boulderfist Buff ID"])
      if not BF then
        return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Boulderfist ID"])
      end
    end
  end
end

ShamAndAwe.priorityTable.name["bf2"] = "Boulderfist (if=buff.boulderfist < gcd | boulderfist.charges > 1.75)"
ShamAndAwe.priorityTable.icon["bf2"] = ShamAndAwe.constants["Boulderfist Icon"]
ShamAndAwe.priorityTable.test["bf2"] =
function ()
  if ShamAndAwe.db.char.talBoulderfist then
    if ShamAndAwe:SpellReady(ShamAndAwe.constants["Boulderfist ID"]) then
      local hasCharges, currentCharges, maxCharges, fracCharges = ShamAndAwe:SpellCharges(ShamAndAwe.constants["Boulderfist ID"])
      local BF = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Boulderfist Buff ID"])
      if fracCharges > 1.75 or not BF then
        return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Boulderfist ID"])
      end
    end
  end
end

ShamAndAwe.priorityTable.name["bf"] = ShamAndAwe.constants["Boulderfist"]
ShamAndAwe.priorityTable.icon["bf"] = ShamAndAwe.constants["Boulderfist Icon"]
ShamAndAwe.priorityTable.test["bf"] =
function ()
  if ShamAndAwe.db.char.talBoulderfist then
    if ShamAndAwe:SpellReady(ShamAndAwe.constants["Boulderfist ID"]) then
      local charges = GetSpellCharges(ShamAndAwe.constants["Boulderfist ID"])
      if charges >= 1 then
        return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Boulderfist ID"])
      end
    end
  end
end

ShamAndAwe.priorityTable.name["rb"] = ShamAndAwe.constants["Rockbiter"]
ShamAndAwe.priorityTable.icon["rb"] = ShamAndAwe.constants["Rockbiter Icon"]
ShamAndAwe.priorityTable.test["rb"] =
function ()
  if not ShamAndAwe.db.char.talBoulderfist then
    if ShamAndAwe:SpellReady(ShamAndAwe.constants["Rockbiter ID"]) then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Rockbiter ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["asc"] = ShamAndAwe.constants["Ascendance"]
ShamAndAwe.priorityTable.icon["asc"] = ShamAndAwe.constants["Ascendance Icon"]
ShamAndAwe.priorityTable.test["asc"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Ascendance ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Ascendance ID"])
  end
end

ShamAndAwe.priorityTable.name["asc_boss"] = "Ascendance (if=boss)"
ShamAndAwe.priorityTable.icon["asc_boss"] = ShamAndAwe.constants["Ascendance Icon"]
ShamAndAwe.priorityTable.test["asc_boss"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Ascendance ID"]) and ShamAndAwe:IsBoss() then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Ascendance ID"])
  end
end

ShamAndAwe.priorityTable.name["ws"] = ShamAndAwe.constants["Windsong"]
ShamAndAwe.priorityTable.icon["ws"] = ShamAndAwe.constants["Windsong Icon"]
ShamAndAwe.priorityTable.test["ws"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Windsong ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Windsong ID"])
  end
end

ShamAndAwe.priorityTable.name["foa_nfoa"] = "Fury of Air (if=!buff.furyofair and maelstrom >= 10)"
ShamAndAwe.priorityTable.icon["foa_nfoa"] = ShamAndAwe.constants["Fury of Air Icon"]
ShamAndAwe.priorityTable.test["foa_nfoa"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Fury of Air ID"]) then
    local FA = UnitAura("player", GetSpellInfo(ShamAndAwe.constants["Fury of Air ID"]))
    local maelstrom = UnitPower("player", "mana")
    if not FA and maelstrom >= 10 then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Fury of Air ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["fb"] = ShamAndAwe.constants["Frostbrand"]
ShamAndAwe.priorityTable.icon["fb"] = ShamAndAwe.constants["Frostbrand Icon"]
ShamAndAwe.priorityTable.test["fb"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Frostbrand ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Frostbrand ID"])
  end
end

ShamAndAwe.priorityTable.name["fb_hs"] = "Frostbrand (if=tal.hailstorm&buff.frostbrand < 4.8)"
ShamAndAwe.priorityTable.icon["fb_hs"] = ShamAndAwe.constants["Frostbrand Icon"]
ShamAndAwe.priorityTable.test["fb_hs"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Frostbrand ID"]) then
    local fb, fb_dur, fb_count = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Frostbrand ID"])
    if ShamAndAwe.db.char.talHailstorm and fb_dur < 4.8 then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Frostbrand ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["fb_nfb"] = "Frostbrand (if=tal.hailstorm&buff.frostbrand < gcd)"
ShamAndAwe.priorityTable.icon["fb_nfb"] = ShamAndAwe.constants["Frostbrand Icon"]
ShamAndAwe.priorityTable.test["fb_nfb"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Frostbrand ID"]) then
    local fb, fb_dur, fb_count = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Frostbrand ID"])
    if ShamAndAwe.db.char.talHailstorm and not fb then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Frostbrand ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["ft"] = ShamAndAwe.constants["Flametongue"]
ShamAndAwe.priorityTable.icon["ft"] = ShamAndAwe.constants["Flametongue Icon"]
ShamAndAwe.priorityTable.test["ft"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Flametongue ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Flametongue ID"])
  end
end

ShamAndAwe.priorityTable.name["ft_nft"] = "Flametongue(if=buff.flametongue < gcd)"
ShamAndAwe.priorityTable.icon["ft_nft"] = ShamAndAwe.constants["Flametongue Icon"]
ShamAndAwe.priorityTable.test["ft_nft"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Flametongue ID"]) then
    local ft, ft_dur, ft_count = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Flametongue Buff ID"])
    if not ft then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Flametongue ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["ft_dur"] = "Flametongue (if=buff.flametongue < 4.8)"
ShamAndAwe.priorityTable.icon["ft_dur"] = ShamAndAwe.constants["Flametongue Icon"]
ShamAndAwe.priorityTable.test["ft_dur"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Flametongue ID"]) then
    local ft, ft_dur, ft_count = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Flametongue Buff ID"])
    if ft_dur < 4.8 then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Flametongue ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["ft_bf"] = "Flametongue (if=tal.boulderfist)"
ShamAndAwe.priorityTable.icon["ft_bf"] = ShamAndAwe.constants["Flametongue Icon"]
ShamAndAwe.priorityTable.test["ft_bf"] =
function ()
  if ShamAndAwe.db.char.talBoulderfist then
    if ShamAndAwe:SpellReady(ShamAndAwe.constants["Flametongue ID"]) then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Flametongue ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["dw"] = ShamAndAwe.constants["Doom Winds"]
ShamAndAwe.priorityTable.icon["dw"] = ShamAndAwe.constants["Doom Winds Icon"]
ShamAndAwe.priorityTable.test["dw"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Doom Winds ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Doom Winds ID"])
  end
end

ShamAndAwe.priorityTable.name["cl"] = ShamAndAwe.constants["Crash Lightning"]
ShamAndAwe.priorityTable.icon["cl"] = ShamAndAwe.constants["Crash Lightning Icon"]
ShamAndAwe.priorityTable.test["cl"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Crash Lightning ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Crash Lightning ID"])
  end
end

ShamAndAwe.priorityTable.name["cl_aoe"] = "Crash Lightning (if=adds >= 3)"
ShamAndAwe.priorityTable.icon["cl_aoe"] = ShamAndAwe.constants["Crash Lightning Icon"]
ShamAndAwe.priorityTable.test["cl_aoe"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Crash Lightning ID"]) then
    local Adds = ShamAndAwe:CountNameplates(8)
    if Adds >= minaoetargets then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Crash Lightning ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["cl_apl"] = "Crash Lightning (if=tal.crashingstorm or adds >= 2 or fs.remaining > 5)"
ShamAndAwe.priorityTable.icon["cl_apl"] = ShamAndAwe.constants["Crash Lightning Icon"]
ShamAndAwe.priorityTable.test["cl_apl"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Crash Lightning ID"]) then
    local fs, fs_cd = ShamAndAwe:SpellCooldown(ShamAndAwe.constants["Feral Spirit ID"])
    local Adds = ShamAndAwe:CountNameplates(8)
    if Adds >= 2 or ShamAndAwe.db.char.talCrashingStorm or fs_cd > 110 then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Crash Lightning ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["ws"] = ShamAndAwe.constants["Windstrike"]
ShamAndAwe.priorityTable.icon["ws"] = ShamAndAwe.constants["Windstrike Icon"]
ShamAndAwe.priorityTable.test["ws"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Windstrike ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Windstrike ID"])
  end
end

ShamAndAwe.priorityTable.name["ss"] = ShamAndAwe.constants["Stormstrike"]
ShamAndAwe.priorityTable.icon["ss"] = ShamAndAwe.constants["Stormstrike Icon"]
ShamAndAwe.priorityTable.test["ss"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Stormstrike ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Stormstrike ID"])
  end
end

ShamAndAwe.priorityTable.name["lb"] = "Lightning Bolt (if=tal.overcharge and maelstrom >= 45)"
ShamAndAwe.priorityTable.icon["lb"] = ShamAndAwe.constants["Lightning Bolt Icon"]
ShamAndAwe.priorityTable.test["lb"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Lightning Bolt ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Lightning Bolt ID"])
  end
end

ShamAndAwe.priorityTable.name["lb_oc"] = "Lightning Bolt (if=tal.overcharge and maelstrom >= 45)"
ShamAndAwe.priorityTable.icon["lb_oc"] = ShamAndAwe.constants["Lightning Bolt Icon"]
ShamAndAwe.priorityTable.test["lb_oc"] =
function ()
  if ShamAndAwe.db.char.talOvercharge then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 45 and ShamAndAwe:SpellReady(ShamAndAwe.constants["Lightning Bolt ID"]) then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Lightning Bolt ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["lb_oc60"] = "Lightning Bolt (if=tal.overcharge and maelstrom >= 60)"
ShamAndAwe.priorityTable.icon["lb_oc60"] = ShamAndAwe.constants["Lightning Bolt Icon"]
ShamAndAwe.priorityTable.test["lb_oc60"] =
function ()
  if ShamAndAwe.db.char.talOvercharge then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 60 and ShamAndAwe:SpellReady(ShamAndAwe.constants["Lightning Bolt ID"]) then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Lightning Bolt ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["ll"] = ShamAndAwe.constants["Lava Lash"]
ShamAndAwe.priorityTable.icon["ll"] = ShamAndAwe.constants["Lava Lash Icon"]
ShamAndAwe.priorityTable.test["ll"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Lava Lash ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Lava Lash ID"])
  end
end

ShamAndAwe.priorityTable.name["ll_hh"] = "Lava Lash (if=tal.hothand and hothand.react)"
ShamAndAwe.priorityTable.icon["ll_hh"] = ShamAndAwe.constants["Lava Lash Icon"]
ShamAndAwe.priorityTable.test["ll_hh"] =
function ()
  if ShamAndAwe.db.char.talHotHand then
    local hh = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Hot Hand ID"])
    if hh and ShamAndAwe:SpellReady(ShamAndAwe.constants["Lava Lash ID"]) then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Lava Lash ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["ll_90"] = "Lava Lash (if=maelstrom >= 90"
ShamAndAwe.priorityTable.icon["ll_90"] = ShamAndAwe.constants["Lava Lash Icon"]
ShamAndAwe.priorityTable.test["ll_90"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Lava Lash ID"]) then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 90 then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Lava Lash ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["ll_120"] = "Lava Lash (if=maelstrom >= 120"
ShamAndAwe.priorityTable.icon["ll_120"] = ShamAndAwe.constants["Lava Lash Icon"]
ShamAndAwe.priorityTable.test["ll_120"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Lava Lash ID"]) then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 120 then
      return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Lava Lash ID"])
    end
  end
end

ShamAndAwe.priorityTable.name["es"] = ShamAndAwe.constants["Earthen Spike"]
ShamAndAwe.priorityTable.icon["es"] = ShamAndAwe.constants["Earthen Spike Icon"]
ShamAndAwe.priorityTable.test["es"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Earthen Spike ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Earthen Spike ID"])
  end
end

ShamAndAwe.priorityTable.name["sun"] = ShamAndAwe.constants["Sundering"]
ShamAndAwe.priorityTable.icon["sun"] = ShamAndAwe.constants["Sundering Icon"]
ShamAndAwe.priorityTable.test["sun"] =
function ()
  if ShamAndAwe:SpellReady(ShamAndAwe.constants["Sundering ID"]) then
    return ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Sundering ID"])
  end
end

-------------------------------
-- Priority Frame functions
-------------------------------

function ShamAndAwe:SetPriorityBackdrop(icon)

  if ShamAndAwe.db.char.priority.enableDisableBorder then

    self.PriorityFrame:SetBackdrop({ bgFile = icon
      })
  else
    --self.PriorityFrame.Icon:Set(icon)

    self.PriorityFrame:SetBackdrop({ bgFile = icon,
        edgeFile = media:Fetch("border", ShamAndAwe.db.char.priority.barborder),
        tile = false, tileSize = 0, edgeSize = 12,
        insets = { left = 2, right = 2, top = 2, bottom = 2 }
      })
    --[[
    if ShamAndAwe.db.char.currentspec == "Enhancement" then
      if ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Enmaelstrom ID"]) then
        self.PriorityFrame:SetBackdropBorderColor(1,0,0,1)
      else
        self.PriorityFrame:SetBackdropBorderColor(1,1,1,1)
      end
    end
    ]]
  end
end

function ShamAndAwe:CreatePriorityFrame()
  self.updatePriority = false
  self.PriorityFrame:SetScale(ShamAndAwe.db.char.priority.scale)
  self.PriorityFrame:SetFrameStrata(ShamAndAwe.db.char.framestratalevel)
  self.PriorityFrame:SetWidth(ShamAndAwe.db.char.priority.fWidth)
  self.PriorityFrame:SetHeight(ShamAndAwe.db.char.priority.fHeight)
  self.PriorityFrame.Icon = _G["SAA_PriorityFrameIcon"]
  self:SetPriorityBackdrop("Interface/Tooltips/UI-Tooltip-Background")
  self.PriorityFrame:SetBackdropColor(0, 0, 0, ShamAndAwe.db.char.priority.alpha);
  self.PriorityFrame:SetMovable(true);
  self.PriorityFrame:RegisterForDrag("LeftButton");
  self.PriorityFrame:EnableMouse(false)
  self.PriorityFrame:SetPoint(ShamAndAwe.db.char.priority.point, ShamAndAwe.db.char.priority.relativeTo, ShamAndAwe.db.char.priority.relativePoint, ShamAndAwe.db.char.priority.xOffset, ShamAndAwe.db.char.priority.yOffset)
  self.PriorityFrame:SetScript("OnDragStart",
    function()
      self.PriorityFrame:StartMoving();
    end );
  self.PriorityFrame:SetScript("OnDragStop",
    function()
      self.PriorityFrame:StopMovingOrSizing();
      self.PriorityFrame:SetScript("OnUpdate", nil);
      self:FinishedMoving(ShamAndAwe.db.char.priority, self.PriorityFrame);
    end );
  self:SetPriorityUpdateScript()
  if not self.PriorityFrame.topText then
    self.PriorityFrame.topText = self.PriorityFrame:CreateFontString(nil, "OVERLAY")
  end
  self.PriorityFrame.topText:SetTextColor(1,1,1,1)
  self.PriorityFrame.topText:SetFont(media:Fetch("font", ShamAndAwe.db.char.barfont), ShamAndAwe.db.char.barfontsize)
  self.PriorityFrame.topText:SetPoint("TOP", self.PriorityFrame, "TOP", 0, ShamAndAwe.db.char.barfontsize + 2)
  self.PriorityFrame.topText:SetText(string.format(L["Next Priority (Set %s)"], ShamAndAwe.db.char.priority.groupnumber))
  self.PriorityFrame.cooldown:SetAllPoints(self.PriorityFrame)
  --self.PriorityFrame.cooldown:SetSwipeTexture('Interface/Cooldown/edge')
  -- self.PriorityFrame.cooldown:SetSwipeColor(0, 0, 0, 0.8)
  self:CreateInterruptPurgeFrames()

  if ShamAndAwe.db.char.priority.show and InCombatLockdown() then
    self.PriorityFrame:Show()
  else
    self.PriorityFrame:Hide()
  end
  if ShamAndAwe.db.char.priority.titleshow then
    self.PriorityFrame.topText:Show()
  else
    self.PriorityFrame.topText:Hide()
  end
  if LBF and LBFgroup then
    LBFgroup:AddButton(self.PriorityFrame)
  end
end

function ShamAndAwe:ResetPriority()
  self.PriorityFrame:ClearAllPoints()
  ShamAndAwe.db.char.priority.point = self.defaults.char.priority.point
  ShamAndAwe.db.char.priority.relativeTo = self.defaults.char.priority.relativeTo
  ShamAndAwe.db.char.priority.relativePoint = self.defaults.char.priority.relativePoint
  ShamAndAwe.db.char.priority.xOffset = self.defaults.char.priority.xOffset
  ShamAndAwe.db.char.priority.yOffset = self.defaults.char.priority.yOffset
  ShamAndAwe.db.char.priority.fWidth = self.defaults.char.priority.fWidth
  ShamAndAwe.db.char.priority.fHeight = self.defaults.char.priority.fHeight
  ShamAndAwe.db.char.priority.scale = self.defaults.char.priority.scale
  ShamAndAwe.db.char.priority.prOption = self.defaults.char.priority.prOption
  self.PriorityFrame:SetPoint(ShamAndAwe.db.char.priority.point, ShamAndAwe.db.char.priority.relativeTo, ShamAndAwe.db.char.priority.relativePoint, ShamAndAwe.db.char.priority.xOffset, ShamAndAwe.db.char.priority.yOffset)
  self:CreatePriorityFrame()
  self:Print(L["priority_reset"])
end

function ShamAndAwe:SetPriorityUpdateScript()
  self.PriorityFrame:SetScript("OnUpdate",
    function()
      if self.updatePriority then
        ShamAndAwe.db.char.priority.previous = ShamAndAwe.db.char.priority.next
        ShamAndAwe:SetNextPriority()
        -- if ShamAndAwe.db.char.priority.next ~= ShamAndAwe.db.char.priority.previous or ShamAndAwe:CheckShamanisticMaelstrom() then
        ShamAndAwe:SetPriorityIcon(ShamAndAwe.db.char.priority.next) -- always set icon to fix the times where it was getting stuck
        -- end
        if ShamAndAwe.db.char.priority.showcooldown then
          local startTime, duration, enabled = GetSpellCooldown(ShamAndAwe.constants["SpellCoolDown"])
          if startTime then
            -- ShamAndAwe:DebugPrint("Cooldown "..startTime.. " - "..duration)

            ShamAndAwe.PriorityFrame.cooldown:SetCooldown(startTime, duration)
          end
        end
      end

      if ShamAndAwe.db.char.priority.show and not ShamAndAwe.db.char.disabled then
        if ShamAndAwe.db.char.priority.showinterrupt and self.PriorityFrame.interrupt then
          local EnemySpell, _, _, EnemySpellIcon, _, _, _, _, EnemySpellInterruptable = UnitCastingInfo("target")
          EnemySpellInterruptable = not EnemySpellInterruptable --reverse since false actually means interruptable
          if not EnemySpell then
            -- OBU.EnemySpellIcon = "Interface/Tooltips/UI-Tooltip-Background"
            self.PriorityFrame.interrupt.frame:SetBackdropColor(0, 0, 0, 0);
          end

          --This section causes high memory allocation?
          self.PriorityFrame.interrupt.frame:SetBackdrop({ bgFile = EnemySpellIcon,
              edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
              tile = false, tileSize = 0, edgeSize = 12,
              insets = { left = 2, right = 2, top = 2, bottom = 2 }
            })
          --------

          self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 1, 1, 1);

          local usable, nomana = IsUsableSpell(ShamAndAwe.constants["Wind Shear ID"]);
          local available = GetSpellCooldown(GetSpellInfo(ShamAndAwe.constants["Wind Shear ID"]))
          if EnemySpellInterruptable then
            if (usable and available == 0) then
              self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(0, 1, 0, 1);
              -- self:SetSubFrameBackdrop(self.PriorityFrame.interrupt.frame, OBU.EnemySpellIcon, 4)
              -- self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1,0,0,1)
            else
              self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 0, 0, 1);
              --self.PriorityFrame.purge.frame:SetBackdropColor(0, 0, 0, 0);
              --self:SetSubFrameBackdrop(self.PriorityFrame.interrupt.frame, OBU.EnemySpellIcon, 12)
              --self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 0, 0, 1); -- set border color to Red if Purge not available
            end
          end
          if not EnemySpell then
            self.PriorityFrame.interrupt.frame:SetBackdropColor(0, 0, 0, 0);
            self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(0, 0, 0, 0);
          end
        end
      end

    end );
end

function ShamAndAwe:SetPriorityIcon(priority)
  --ShamAndAwe:DebugPrint("setting icon to "..priority)
  if InCombatLockdown() or priority == "none" then
    local icon = self.priorityTable.icon[priority]
    -- if self:CheckWindShear() then
    -- self:SetSubFrameBackdrop(self.PriorityFrame.purge.frame, ShamAndAwe.constants["Heroic Strike Icon"], 4)
    -- else
    -- self:SetSubFrameBackdrop(self.PriorityFrame.purge.frame, "Interface/Tooltips/UI-Tooltip-Background", 4)
    -- self.PriorityFrame.purge.frame:SetBackdropColor(0, 0, 0, 0);
    -- end
    self:SetPriorityBackdrop(icon)
  end
end

function ShamAndAwe:CheckWindShear()
  if ShamAndAwe.db.char.hsshow then
    -- local UL = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Ultimatum"])
    -- if UL then
    -- return true
    -- else
    -- local avail = ShamAndAwe:SpellAvailable(ShamAndAwe.constants["Heroic Strike"])
    -- local usable, nomana = IsUsableSpell(ShamAndAwe.constants["Heroic Strike"])
    -- if usable and avail then
    -- local TargetSubTwenty = ShamAndAwe:TargetSubTwenty()
    -- local CS, CSduration = ShamAndAwe:GetDebuffInfo(ShamAndAwe.constants["Colossus Smash"])
    -- local TFB, TFBduration, TFBcount = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Taste for Blood"])
    -- local BC = ShamAndAwe:GetBuffInfo(ShamAndAwe.constants["Battle Cry"])
    -- local maelstrom = UnitPower("player", "mana")

    -- if maelstrom >= ShamAndAwe.db.char.maelstromThreshold then
    -- return true
    -- elseif ShamAndAwe.db.char.currentspec == "Enhancement" then
    --Enhancement
    -- if not TargetSubTwenty then
    -- -- Target below 20% health
    -- if CS then
    -- if maelstrom >= 40 then
    -- return true
    -- end
    -- elseif (DC and CS) then
    -- if maelstrom >= 30 then
    -- return true
    -- end
    -- end
    -- end
    --End Enhancement
    -- elseif ShamAndAwe.db.char.currentspec == "Arms" then
    --Arms
    -- if not TargetSubTwenty then
    -- if ((TFB and TFBduration <=2) or (TFBcount == 5) or (TFB and CSduration <= 2) or BC) and CS then
    -- return true
    -- end
    -- end
    -- end
    -- end
    -- end
  end
end

function ShamAndAwe:SetNextPriority()
  -- force update of stacks count to ensure priority shown correctly
  -- ShamAndAwe.db.char.msstacks = ShamAndAwe:GetMaelstromInfo()
  -- if ShamAndAwe.db.char.priority.combopoints then
  -- self:ShowComboPoints()
  -- end
  ShamAndAwe.db.char.priority.next = "none"
  for index = 1, 30 do
    if ShamAndAwe.db.char.priority.prOption[index] and self.priorityTable.test[ShamAndAwe.db.char.priority.prOption[index]] then -- verify that the option actually exists
      if self.priorityTable.test[ShamAndAwe.db.char.priority.prOption[index]]() then
        ShamAndAwe.db.char.priority.next = ShamAndAwe.db.char.priority.prOption[index]
        return -- we need to break out of the routine as we have found the top priority
      end
    end
  end
end

function ShamAndAwe:TargetSubTwenty()
  local dechp
  local perchp
  local calchp
  local currenthp = UnitHealth("target")
  local totalhp = UnitHealthMax("target")
  if currenthp == 0 then
    dechp = 0
    perchp = 0
  elseif currenthp == totalhp then
    perchp = 100
  else
    calchp = currenthp / totalhp
    dechp = calchp * 100
    perchp = math.ceil(dechp)
  end

  if perchp <= 20 then
    return true
  else
    return false
  end
end

function ShamAndAwe:SelfSubTwenty()
  local selfcurrenthp = UnitHealth("player")
  local selftotalhp = UnitHealthMax("player")
  local selfdechp
  local selfperchp
  local selfcalchp
  if selfcurrenthp == 0 then
    selfdechp = 0
    selfperchp = 0
  elseif currenthp == totalhp then
    selfperchp = 100
  else
    local selfcalchp = selfcurrenthp / selftotalhp
    local selfdechp = selfcalchp * 100
    selfperchp = math.ceil(dechp)
  end

  if selfperchp <= 20 then
    return true
  else
    return false
  end
end

function ShamAndAwe:CreateInterruptPurgeFrames()

  local width = ShamAndAwe.db.char.priority.fWidth / 1.6
  local height = ShamAndAwe.db.char.priority.fHeight / 1.6

  if not self.PriorityFrame.interrupt then
    self.PriorityFrame.interrupt = {}
    self.PriorityFrame.interrupt.frame = CreateFrame("Frame", "SAA_PriorityInterruptFrame", self.PriorityFrame)
  end
  local interruptFrame = self.PriorityFrame.interrupt.frame
  interruptFrame:SetScale(ShamAndAwe.db.char.priority.scale)
  interruptFrame:SetFrameStrata("BACKGROUND")
  interruptFrame:SetWidth(width)
  interruptFrame:SetHeight(height)

  --ShamAndAwe:SetTexture("Interface/Tooltips/UI-Tooltip-Background", false)

  self:SetSubFrameBackdrop(interruptFrame, "Interface/Tooltips/UI-Tooltip-Background", 4)
  interruptFrame:SetBackdropColor(0, 0, 0, 0);
  interruptFrame:SetPoint("BOTTOMLEFT", self.PriorityFrame, "BOTTOMRIGHT", 0 , 0)
  if ShamAndAwe.db.char.priority.showinterrupt then
    interruptFrame:Show()
  else
    interruptFrame:Hide()
  end

  if not self.PriorityFrame.purge then
    self.PriorityFrame.purge = {}
    self.PriorityFrame.purge.frame = CreateFrame("Frame", "SAA_PrioritypurgeFrame", self.PriorityFrame)
  end

  local purgeFrame = self.PriorityFrame.purge.frame
  purgeFrame:SetScale(ShamAndAwe.db.char.priority.scale)
  purgeFrame:SetFrameStrata("BACKGROUND")
  purgeFrame:SetWidth(width)
  purgeFrame:SetHeight(height)
  self:SetSubFrameBackdrop(purgeFrame, "Interface/Tooltips/UI-Tooltip-Background", 4)
  purgeFrame:SetBackdropColor(0, 0, 0, 0);
  purgeFrame:SetPoint("BOTTOMLEFT", self.PriorityFrame, "BOTTOMRIGHT", 0 , 0)
  if ShamAndAwe.db.char.priority.showpurge then
    purgeFrame:Show()
  else
    purgeFrame:Hide()
  end
end

--[[
function ShamAndAwe:SetSubFrameBackdrop(subFrame, icon, edgeSize)
  subFrame:SetBackdrop({ bgFile = icon,
      edgeSize = edgeSize
    })
end
]]
function ShamAndAwe:SetSubFrameBackdrop(subFrame, icon, edgeSize)
  subFrame:SetBackdrop({ bgFile = icon,
      tile = false, tileSize = 0, edgeSize = edgeSize

    })
end
